let Products = [];
var UnicornSearch = function (url) {
  this.url = url;
  this.amazon = amazon(url);
  this.domain = this.amazon.hostname; //获取当前站点的网址
  this.products = this.newProductsCollection();
  this.nowProducts = this.newProductsCollection();
  this.xhrPool = [];
  this.timestamp = Date.now(); //获取当前的毫秒数
  this.sales = ""; //设置筛选销量变量
  this.revenue = "";
  this.lens = 0;
  this.ajaxCompleteCallbackFunction = "";

  if (!this.amazon.supported) {
    //是否是支持的站点
    return this;
  }
  if (helper.regex(this.amazon.regex.search.url.page) != "") {
    this.url = this.url.replace(
      helper.regex(this.amazon.regex.search.url.page),
      ""
    ); //去除页码
  }
  this.favorite = 0;
  this.variationsArr = [];
  this.keywords = helper.regex(this.amazon.regex.search.url.keywords, url); //获取关键词
  if (!this.keywords)
    this.keywords = helper.regex(this.amazon.regex.search.url.keywords1, url); //获取关键词
  this.variations = "";
  this.pageNext = 1;
  this.sponsored = "";
  (this.date = ""), (this.shopByCa = 0);
  this.clearFilters(); //清空筛选
  this.multiPage = [];
  if (this.url != "") {
    if (this.url.indexOf("/dp/") > -1) {
      this.multiProducts();
    } else {
      this.scrapeProducts();
    } //抓取产品数据
  }
};

// Set up and return products array with helper functions
UnicornSearch.prototype.newProductsCollection = function () {
  var products = [];
  products.get = function () {
    var productArray = [];
    $.each(this, function (index, product) {
      productArray.push(product.get()); //给每个product添加get方法
    });
    return productArray;
  };

  products.setAjaxCompleteCallback = function (callback) {
    //给每个product添加属性
    $.each(this, function (index, product) {
      product.ajaxCompleteCallback = callback;
    });
  };
  return products;
};

UnicornSearch.prototype.xhrAbortAll = function () {
  var self = this;

  this.xhrPool.each(function (i, jqXHR) {
    jqXHR.abort();
    self.xhrPool.splice(i, 1);
  });
};

UnicornSearch.prototype.set = function (search) {
  //设置数据
  this.url = search.url;
  this.timestamp = search.timestamp;
  this.favorite = search.favorite;
  this.domain = search.domain;
  this.keywords = search.keywords;
  this.pageNext = search.pageNext;
  this.filters = search.filters;
  //	this.multiPage = search.multiPage;
  this.products = this.newProductsCollection();
  this.nowProducts = this.newProductsCollection();
  var self = this;

  var delay = 0;
  $.each(search.products, function (index, product) {
    var newProduct = new UnicornProduct(product);
    setTimeout(newProduct.getProductDetails.bind(newProduct), 50 * delay++);

    newProduct.exportExcelCallback = self.exportExcelCallback;
    newProduct.ajaxCompleteCallback = self.ajaxCompleteCallback;
    self.products.push(newProduct);
    self.nowProducts.push(newProduct);
    newProduct = null;
  });
};

UnicornSearch.prototype.get = async function () {
  let data = []
  for (let index = 0; index < 1000; index++) {
    if (index <= this.pageNext) {
      let value = await localforage.getItem('product' + this.domain + index);
      if (value) {
        data = data.concat(JSON.parse(value))
      }
    }
  }
  data.forEach((item, index) => {
    item.rank = index
  })
  //获取信息
  return {
    url: this.url,
    timestamp: this.timestamp,
    favorite: this.favorite,
    domain: this.domain,
    keywords: this.keywords,
    multiPage: this.multiPage,
    products: data,
    pageNext: this.pageNext,
    filters: this.filters,
  };
};

UnicornSearch.prototype.clearFilters = function () {
  //清空筛选
  this.filters = {
    price: { min: "", max: "" },
    salesrank: { min: "", max: "" },
    variations: { min: "", max: "" },
    sellers: { min: "", max: "" },
    sales: { min: "", max: "" },
    revenue: { min: "", max: "" },
    reviews: { min: "", max: "" },
    rating: { min: "", max: "" },
    hideSponsored: 0,
    onlyFba: 0,
  };
};

var indexL = 0;
UnicornSearch.prototype.scrapeProducts = function (refresh, state) {
  //抓取数据
  if (refresh == "refresh") this.url = document.URL; //如果是刷新，url等于当前网址
  if (this.url == "") return; //如果页面为空白地址，则跳出不执行
  var self = this;
  var url = this.url;
  var regex = this.amazon.regex;
  let rankLength = self.products[self.products.length - 2] ? self.products[self.products.length - 2].rank : 0;
  self.nowProducts = [];
  self.products = [];
  self.nowProducts = this.newProductsCollection(); //给变量赋上对象
  self.products = this.newProductsCollection(); //给变量赋上对象
  if (refresh == "refresh") {
    self.keywords = helper.regex(this.amazon.regex.search.url.keywords, url); //获取关键词
  } else {
    if (this.pageNext > 1) {
      if (
        this.url.indexOf("Best-Sellers") != -1 ||
        this.url.indexOf("gp/bestsellers") != -1
      ) {
        url = $(".a-last a")[0].href;
        // console.log($(".a-last a")[0].href, "====1");
      } else {
        url += "&page=" + this.pageNext; //给URL改变成下一页
      }
    }
  }
  $.ajax({
    url: url,
    method: "GET",
    timeout: 10000, //by ouyang
    cache: false,
    beforeSend: function (jqXHR) {
      self.xhrPool.push(jqXHR); //给数组设置值
    },
    complete: function (jqXHR) {
      var index = self.xhrPool.indexOf(jqXHR);
      if (index > -1) {
        self.xhrPool.splice(index, 1); //删除数组中添加的值
      }
      //如果ajaxCompleteCallback为空，this.ajaxCompleteCallbackFunction 赋值；products.setAjaxCompleteCallback(callback)赋值
      if (self.ajaxCompleteCallback != "")
        self.ajaxCompleteCallback(
          "_history " + self.domain + " " + self.keywords
        );
      //    console.log('%cUS: product page scraping completed:', 'color: blue');
      //    console.timeEnd('from start');
    },
    error: function (jqXHR, textStatus, errorThrown) {
      //by ouyang
      var index = self.xhrPool.indexOf(jqXHR);
      if (index > -1) {
        //删除数组中添加的值
        self.xhrPool.splice(index, 1);
      }
    },
    success: function (html) {
      $("#uniPopupPro #refresh-search").css("animation", "none");
      // redirect to captcha if search page returns captcha
      // otherwise the widget will be stuck in "analyzing..." state
      if (helper.isCaptchaPage(html)) {
        document.location.href = url;
      }
      var cleanHTML = helper.clearHTMLComments(html); //去掉注释
      var scrapeDelay = 0,
        rank = 0,
        productList;
      // var len = self.products.length;
      // if(self.keywords.indexOf("https:")!==-1){
      //   const keywords = helper.regex(regex.product.product_Details.keywords, html);
      //   if(keywords){
      //     self.keywords = keywords;
      //   }
      // }
      rank = Number(rankLength) + rank;
      if (url) {
        if (url.indexOf("gp/bestsellers") != -1 || url.match(/Best-Sellers/i)) {
          if (
            url.indexOf("pd_zg_") !== -1 ||
            document.getElementsByClassName("zg-item-immersion").length
          ) {
            productList = regex.search.html.products2;
          } else if (url.indexOf("nav_cs_bestsellers") !== -1) {
            productList = regex.search.html.multiList;
          } else {
            productList = regex.search.html.products3;
          }
        } else {
          productList = regex.search.html.products;
        }
      }
      if (!productList.exec(cleanHTML)) {
        if (url.indexOf("gp/bestsellers") != -1 || url.match(/Best-Sellers/i)) {
          if (url.match(/best-sellers|pd_zg_/i)) {
            productList = regex.search.html.products2;
          } else if (
            url.indexOf("nav_cs_bestsellers") !== -1 ||
            document.getElementsByClassName("a-carousel-card").length
          ) {
            productList = regex.search.html.multiList;
          } else {
            productList = regex.search.html.products3;
          }
        } else {
          productList = regex.search.html.products1;
        }
      }
      if (url.indexOf("gp/bestsellers") != -1 || url.match(/Best-Sellers/i)) {
        let list = null;
        console.log(regex.search.html.bestsellers,
          url.indexOf("nav_cs_bestsellers"),
          document.getElementsByClassName("zg-item-immersion").length)
        if (
          regex.search.html.bestsellers &&
          document.getElementsByClassName("zg-item-immersion").length
        ) {
          list = cleanHTML
            .match(regex.search.html.bestsellers)[0]
            .match(productList);
        } else {
          list = cleanHTML.match(productList);
        }
        list.forEach((div) => {
          let product = null;

          var link = self.amazon.parse(
            helper.regex(regex.search.html.link2, div)
          ).url; //获取当前asin的链接
          //      if(!link) link = self.amazon.parse(helper.regex(regex.search.html.link, div)).url;
          var exists = 0;
          if (/".*?=$/g.exec(link)) link = link.replace(/".*?=$/g, "");

          //      if (refresh == undefined){
          //        $.each(self.products, function (index, product) {
          //          if (product.rank == rank) {//如果product.rank有值等于rank，跳出
          //            exists = 1;
          //            return false;
          //          }
          //        });
          //      }

          if (((refresh == undefined && !exists) || refresh == "refresh") && link.length > 40) {
            rank++;
            product = new UnicornProduct(link);
            // setTimeout(
            //   product.getProductDetails.bind(product),
            //   50 * scrapeDelay++
            // ); //获取详情页的sales revenue
            //给当前的asin对象赋值
            product.exportExcelCallback = self.exportExcelCallback;
            product.ajaxCompleteCallback = self.ajaxCompleteCallback;
            product.domain = self.domain;
            product.keywords = self.keywords;
            product.rank = rank;
            if (helper.regex(regex.product.html.sponsored, div))
              product.sponsored = "sp";
            else product.sponsored = "";
            //        if (link.indexOf('redirect.html') == -1) {//获取title
            //          product.title = helper.regex(regex.search.html.title, div);
            //        }
            //        else {
            //          product.title = "(Sponsored) " + helper.regex(regex.search.html.title, div);
            //        }
            product.title = helper.regex(regex.search.html.title3, div);
            if (!product.title)
              product.title = helper.regex(regex.search.html.title1, div);
            product.currency = self.amazon.currency; //获取当前站点的货币符号
            product.asin = helper.regex(regex.search.html.asin, div); //获取asin
            product.brand = helper.regex(regex.search.html.brand, div); //brand
            product.image = helper.regex(regex.search.html.image, div); //image
            product.price = helper.regex(regex.search.html.price, div); //获取price
            if (!product.price)
              product.price = helper.regex(regex.search.html.price1, div);
            if (!product.price && self.amazon.hostname == "www.amazon.de")
              product.price = helper.regex(regex.search.html.price2, div);
            if (self.amazon.hostname == "www.amazon.co.jp") {
              //如果是日本站点，获取category
              product.category = helper.regex(regex.search.html.category, div);
            }
            product.rating = helper.regex(regex.search.html.rating, div); //获取rating
            if (product.rating == "") {
              product.rating = helper.regex(regex.search.html.rating1, div); //获取rating
            }
            product.reviews = helper.regex(regex.search.html.reviews, div); //获取reviews
            //if(!product.reviews) product.reviews = helper.regex(regex.search.html.reviews4, div);//获取reviews
            //        if(product.reviews == "") product.reviews = 0;
            //        if(product.rating == "") product.rating = 0;
            product.sellersUsed = helper.regex(
              regex.search.html.sellersUsed,
              div
            ); //获取sellersUsed
            product.sellersNew = helper.regex(
              regex.search.html.sellersNew,
              div
            ); //获取sellersNew
            if (product.sellersNew == "") product.sellersNew = 1; //当sellersNew为空时的赋值
            if (product.sellersUsed == "") product.sellersUsed = "-"; //当sellersUsed为空时的赋值
            //一下站点需要对价钱进行处理
            if (
              self.amazon.hostname == "www.amazon.de" ||
              self.amazon.hostname == "www.amazon.fr" ||
              self.amazon.hostname == "www.amazon.es" ||
              self.amazon.hostname == "www.amazon.it"
            ) {
              product.price = product.price.replace(".", "");
              product.price = product.price.replace(/\s/g, "");
              product.price = helper.replaceDot(product.price);
              if (product.rating != 0)
                product.rating = helper.replaceDot(product.rating);
            }
            if (self.amazon.hostname == "www.amazon.com") {
              //以下站点对brand的处理
              product.brand =
                product.brand == ""
                  ? helper.regex(regex.search.html.brand2, div)
                  : product.brand;
              product.brand =
                product.brand == ""
                  ? helper.regex(regex.search.html.brand3, div)
                  : product.brand;
              product.brand =
                product.brand == ""
                  ? helper.regex(regex.search.html.brand4, div)
                  : product.brand;
            }
            self.products.push(product); //获取全部的products的数据
            self.nowProducts.push(product); //获取全部的products的数据
            // console.log("商品详情1",product)
            setTimeout(
              product.getProductDetails.bind(product),
              50 * scrapeDelay++
            ); //获取详情页的sales revenue
          }
        });
      } else {
        while ((match = productList.exec(cleanHTML))) {
          let link = null;
          let product = null;
          //将cleanHTML更具正则匹配
          var div = match[1];
          // console.log(regex.search.html.link2.exec(div)&&regex.search.html.link2.exec(div)[1])
          // console.log(helper.regex(regex.search.html.link2, div))
          if (
            helper.regex(regex.search.html.link, div).indexOf("https:") === -1
          ) {
            link = self.amazon.parse(
              helper.regex(regex.search.html.link3, div)
            ).url; //获取当前asin的链接
          } else {
            link = helper.regex(regex.search.html.link2, div); //获取当前asin的链接
          }
          // if(link === `${location.origin}/`&& regex.search.html.link2.exec(div)){
          //   console.log(regex.search.html.link2.exec(div))
          //   link = self.amazon.parse(
          //     regex.search.html.link2.exec(div)[1]
          //   ).url;
          // }
          //      if(!link) link = self.amazon.parse(helper.regex(regex.search.html.link, div)).url;
          var exists = 0;
          if (/".*?=$/g.exec(link)) link = link.replace(/".*?=$/g, "");

          //      if (refresh == undefined){
          //        $.each(self.products, function (index, product) {
          //          if (product.rank == rank) {//如果product.rank有值等于rank，跳出
          //            exists = 1;
          //            return false;
          //          }
          //        });
          //      }
          if (((refresh == undefined && !exists) || refresh == "refresh") && link.length > 40) {
            rank++;
            if (div.indexOf("Shop by Category") > -1) {
              indexL = indexL - 1;
              if (rank + indexL <= 0) {
                indexL = 0;
              }
              rank = rank + indexL;
              continue;
            }
            if (div.indexOf("Expert recommendations") > -1) {
              indexL = indexL - 1;
              if (rank + indexL <= 0) {
                indexL = 0;
              }
              rank = rank + indexL;
              continue;
            }
            product = new UnicornProduct(link);
            product.exportExcelCallback = self.exportExcelCallback;
            // setTimeout(
            //   product.getProductDetails.bind(product, state),
            //   50 * scrapeDelay++
            // ); //获取详情页的sales revenue
            //给当前的asin对象赋值
            product.ajaxCompleteCallback = self.ajaxCompleteCallback;
            product.domain = self.domain;
            product.keywords = self.keywords;
            product.rank = rank;
            if (helper.regex(regex.product.html.sponsored, div))
              product.sponsored = "sp";
            else product.sponsored = "";
            //        if (link.indexOf('redirect.html') == -1) {//获取title
            //          product.title = helper.regex(regex.search.html.title, div);
            //        }
            //        else {
            //          product.title = "(Sponsored) " + helper.regex(regex.search.html.title, div);
            //        }
            product.title = helper.regex(regex.search.html.title, div);
            if (!product.title)
              product.title = helper.regex(regex.search.html.title1, div);

            product.currency = self.amazon.currency; //获取当前站点的货币符号
            product.asin = helper.regex(regex.search.html.asin, div); //获取asin
            product.brand = helper.regex(regex.search.html.brand, div); //brand
            product.image = helper.regex(regex.search.html.image, div); //image
            product.price = helper.regex(regex.search.html.price, div); //获取price
            if (!product.price)
              product.price = helper.regex(regex.search.html.price1, div);
            if (!product.price && self.amazon.hostname == "www.amazon.de")
              product.price = helper.regex(regex.search.html.price2, div);
            if (self.amazon.hostname == "www.amazon.co.jp") {
              //如果是日本站点，获取category
              product.category = helper.regex(regex.search.html.category, div);
            }
            product.rating = helper.regex(regex.search.html.rating, div); //获取rating
            if (product.rating == "") {
              product.rating = helper.regex(regex.search.html.rating1, div); //获取rating
            }
            product.reviews = helper.regex(regex.search.html.reviews, div); //获取reviews
            //if(!product.reviews) product.reviews = helper.regex(regex.search.html.reviews4, div);//获取reviews
            //        if(product.reviews == "") product.reviews = 0;
            //        if(product.rating == "") product.rating = 0;
            product.sellersUsed = helper.regex(
              regex.search.html.sellersUsed,
              div
            ); //获取sellersUsed
            product.sellersNew = helper.regex(
              regex.search.html.sellersNew,
              div
            ); //获取sellersNew
            if (product.sellersNew == "") product.sellersNew = 1; //当sellersNew为空时的赋值
            if (product.sellersUsed == "") product.sellersUsed = "-"; //当sellersUsed为空时的赋值
            //一下站点需要对价钱进行处理
            if (
              self.amazon.hostname == "www.amazon.de" ||
              self.amazon.hostname == "www.amazon.fr" ||
              self.amazon.hostname == "www.amazon.es" ||
              self.amazon.hostname == "www.amazon.it"
            ) {
              product.price = product.price.replace(".", "");
              product.price = product.price.replace(/\s/g, "");
              product.price = helper.replaceDot(product.price);
              if (product.rating != 0)
                product.rating = helper.replaceDot(product.rating);
            }
            if (self.amazon.hostname == "www.amazon.com") {
              //以下站点对brand的处理
              product.brand =
                product.brand == "" ? helper.regex(regex.search.html.brand2, div) : product.brand;
              product.brand = product.brand == "" ? helper.regex(regex.search.html.brand3, div) : product.brand;
              product.brand = product.brand == "" ? helper.regex(regex.search.html.brand4, div) : product.brand;
            }
            self.products.push(product); //获取全部的products的数据
            self.nowProducts.push(product); //获取全部的products的数据
            // console.log(Products.push(product))
            // console.log("商品详情2",product)
            setTimeout(
              product.getProductDetails.bind(product, state),
              50 * scrapeDelay++
            ); //获取详情页的sales revenue
            product = null;
          }
        } // end of while
      }

      if (self.products.length > 0) {
        var lastChild;
        if (self.pageNext > 1) {
          var divs = document.createElement("div");
          divs.innerHTML = html;
          lastChild = divs.getElementsByClassName("s-result-list sg-row")[0];
          self.lens = lastChild ? lastChild.children.length : 0 + self.lens;
        } else {
          lastChild = document.getElementsByClassName(
            "s-result-list sg-row"
          )[0];
          self.lens = lastChild ? lastChild.children.length : 0;
        }
        //    	console.log(self.products.length,self.lens)
        if (self.products.length < self.lens) {
          let product1 = null;
          lastChild = lastChild.lastElementChild.outerHTML;
          rank = self.products.length * 1;
          if (/".*?=$/g.exec(links)) links = links.replace(/".*?=$/g, "");
          var links = self.amazon.parse(
            helper.regex(regex.search.html.link, lastChild)
          ).url; //获取当前asin的链接
          product1 = new UnicornProduct(links);
          // setTimeout(
          //   product1.getProductDetails.bind(product1),
          //   50 * scrapeDelay++
          // ); //获取详情页的sales revenue
          //给当前的asin对象赋值
          product1.ajaxCompleteCallback = self.ajaxCompleteCallback;
          product1.domain = self.domain;
          product1.keywords = self.keywords;
          product1.rank = rank;
          if (helper.regex(regex.product.html.sponsored, lastChild))
            product1.sponsored = "sp";
          else product1.sponsored = "";
          product1.title = helper.regex(regex.search.html.title, lastChild);
          if (!product1.title)
            product1.title = helper.regex(regex.search.html.title1, lastChild);
          product1.currency = self.amazon.currency; //获取当前站点的货币符号
          product1.asin = helper.regex(regex.search.html.asin, lastChild); //获取asin
          product1.brand = helper.regex(regex.search.html.brand, lastChild); //brand
          product1.image = helper.regex(regex.search.html.image, lastChild); //image

          product1.price = helper.regex(regex.search.html.price, lastChild); //获取price
          if (!product1.price)
            product1.price = helper.regex(regex.search.html.price1, lastChild);
          if (!product1.price && self.amazon.hostname == "www.amazon.de")
            product1.price = helper.regex(regex.search.html.price2, lastChild);
          if (self.amazon.hostname == "www.amazon.co.jp") {
            //如果是日本站点，获取category
            product1.category = helper.regex(
              regex.search.html.category,
              lastChild
            );
          }
          product1.rating = helper.regex(regex.search.html.rating, lastChild); //获取rating
          if (
            product1.rating == "" &&
            self.amazon.hostname == "www.amazon.de"
          ) {
            product1.rating = helper.regex(regex.search.html.rating1, lastChild); //获取rating
          }
          product1.reviews = helper.regex(regex.search.html.reviews, lastChild); //获取reviews
          if (!product1.reviews)
            product1.reviews = helper.regex(
              regex.search.html.reviews4,
              lastChild
            ); //获取reviews
          product1.sellersUsed = helper.regex(
            regex.search.html.sellersUsed,
            lastChild
          ); //获取sellersUsed
          product1.sellersNew = helper.regex(
            regex.search.html.sellersNew,
            lastChild
          ); //获取sellersNew
          if (product1.sellersNew == "") product1.sellersNew = 1; //当sellersNew为空时的赋值
          if (product1.sellersUsed == "") product1.sellersUsed = "-"; //当sellersUsed为空时的赋值
          //一下站点需要对价钱进行处理
          if (
            self.amazon.hostname == "www.amazon.de" ||
            self.amazon.hostname == "www.amazon.fr" ||
            self.amazon.hostname == "www.amazon.es" ||
            self.amazon.hostname == "www.amazon.it"
          ) {
            product1.price = product1.price.replace(".", "");
            product1.price = product1.price.replace(/\s/g, "");
            product1.price = helper.replaceDot(product1.price);
            if (product1.rating != 0)
              product1.rating = helper.replaceDot(product1.rating);
          }
          if (self.amazon.hostname == "www.amazon.com") {
            //以下站点对brand的处理
            product1.brand =
              product1.brand == ""
                ? helper.regex(regex.search.html.brand2, lastChild)
                : product1.brand;
            product1.brand =
              product1.brand == ""
                ? helper.regex(regex.search.html.brand3, lastChild)
                : product1.brand;
            product1.brand =
              product1.brand == ""
                ? helper.regex(regex.search.html.brand4, lastChild)
                : product1.brand;
          }
          self.products.push(product1); //获取全部的products的数据
          self.nowProducts.push(product1); //获取全部的products的数据
          // console.log("商品详情2",product)
          setTimeout(
            product1.getProductDetails.bind(product1),
            50 * scrapeDelay++
          ); //获取详情页的sales revenue
          product1 = null;
        }
      }
      if (refresh == undefined) self.pageNext++;
    },
  });
};

UnicornSearch.prototype.multiProducts = function (refresh) {
  if (refresh == "refresh") this.url = document.URL; //如果是刷新，url等于当前网址

  if (this.url == "") return; //如果页面为空白地址，则跳出不执行
  var self = this;
  var url = this.url;
  var regex = this.amazon.regex,
    mainProduct = {};
  if (refresh == "refresh") {
    self.multiPage = this.newProductsCollection(); //给变量赋上对象
    //  self.keywords = helper.regex(this.amazon.regex.search.url.keywords, url);//获取关键词
  }
  $.ajax({
    url: url,
    method: "GET",
    timeout: 10000, //by ouyang
    cache: false,
    beforeSend: function (jqXHR) {
      self.xhrPool.push(jqXHR); //给数组设置值
    },
    complete: function (jqXHR) {
      var index = self.xhrPool.indexOf(jqXHR);
      if (index > -1) {
        self.xhrPool.splice(index, 1); //删除数组中添加的值
      }
      //如果ajaxCompleteCallback为空，this.ajaxCompleteCallbackFunction 赋值；products.setAjaxCompleteCallback(callback)赋值
      if (self.ajaxCompleteCallback != "")
        self.ajaxCompleteCallback(
          "_history " + self.domain + " " + self.keywords
        );
      //    console.log('%cUS: product page scraping completed:', 'color: blue');
      //    console.timeEnd('from start');
    },
    error: function (jqXHR, textStatus, errorThrown) {
      //by ouyang
      var index = self.xhrPool.indexOf(jqXHR);
      if (index > -1) {
        //删除数组中添加的值
        self.xhrPool.splice(index, 1);
      }
    },
    success: function (html) {
      $("#uniPopupPro #refresh-search").css("animation", "none");
      // redirect to captcha if search page returns captcha
      // otherwise the widget will be stuck in "analyzing..." state
      if (helper.isCaptchaPage(html)) {
        document.location.href = url;
      }
      var cleanHTML = helper.clearHTMLComments(html); //去掉注释
      var rank = 1;

      var scrapeDelay = 0;
      var products = new UnicornProduct(url);

      mainProduct = products.extractProductDetails(cleanHTML);
      mainProduct = Object.assign(products, mainProduct);
      var uri =
        "domain=" +
        self.domain +
        "&category=" +
        encodeURIComponent(mainProduct.category.replace("&amp;", "&")) +
        "&rank=" +
        encodeURIComponent(helper.toNumber(mainProduct.salesRank));

      $.ajax({
        url: this.amazon.URL[this.amazon.env] + "unicorn/data" + uri,
        method: "GET",
        cache: false,
        beforeSend: function (jqXHR) {
          self.xhrPool.push(jqXHR);
        },
        complete: function (jqXHR) {
          var index = self.xhrPool.indexOf(jqXHR);
          if (index > -1) {
            self.xhrPool.splice(index, 1);
          }

          if (self.ajaxCompleteCallback != "")
            self.ajaxCompleteCallback(
              "_history " + self.domain + " " + self.keywords
            );
        },
        error: function (jqXHR, textStatus, errorThrown) {
          //by ouyang
          var index = self.xhrPool.indexOf(jqXHR);
          if (index > -1) {
            self.xhrPool.splice(index, 1);
          }
        },
        success: function (data) {
          if ($.isNumeric(data)) {
            //如果有数据
            mainProduct.sales = data; //销量为返回的data值
            mainProduct.revenue = Math.round(Number(mainProduct.price) * data); //收入为价格*销量
          } else {
            mainProduct.sales = "-";
            mainProduct.revenue = "-";
          }
        },
      });
      mainProduct.title = helper.trimStr(
        helper.regex(regex.product.html.title, cleanHTML)
      );
      mainProduct.brand = helper.regex(
        regex.search.html.brandDetail,
        cleanHTML
      );
      mainProduct.reviews = helper.regex(regex.search.html.reviews3, cleanHTML);
      mainProduct.rank = rank;
      mainProduct.currency = self.amazon.currency;
      self.multiPage.push(mainProduct);
      let match = null;
      while ((match = regex.search.html.multiList.exec(cleanHTML))) {
        var list = match[1];
        // console.log(list);
        rank++;
        var link = self.amazon.parse(
          helper.regex(regex.search.html.link2, list)
        ).url;
        if (!link)
          link = self.amazon.parse(
            helper.regex(regex.search.html.link, list)
          ).url;
        if (!link)
          link = self.amazon.parse(
            helper.regex(regex.search.html.link1, list)
          ).url;
        var price = helper.regex(regex.search.html.price, list);
        if (!price) price = helper.regex(regex.search.html.price1, list);
        if ((refresh == undefined || refresh == "refresh") && link.length > 40) {
          var product = new UnicornProduct(link);
          //给当前的asin对象赋值
          product.ajaxCompleteCallback = self.ajaxCompleteCallback;
          product.domain = self.domain;
          product.keywords = self.keywords;
          product.rank = rank;
          if (helper.regex(regex.product.html.sponsored, list))
            product.sponsored = "sp";
          else product.sponsored = "";
          if (link.indexOf("redirect.html") == -1) {
            //获取title
            product.title = helper.regex(regex.search.html.title, list);
          } else {
            product.title =
              "(Sponsored) " + helper.regex(regex.search.html.title, list);
          }
          product.title = helper.regex(regex.search.html.titles, list);
          if (!product.title)
            product.title = helper.regex(regex.search.html.title, list);
          product.currency = self.amazon.currency; //获取当前站点的货币符号
          product.asin = helper.regex(regex.search.html.asin, list); //获取asin
          product.image = helper.regex(regex.search.html.image, list); //image
          product.price = price; //获取price
          product.rating = helper.regex(regex.search.html.rating, list); //获取rating
          product.reviews = helper.regex(regex.search.html.reviews, list); //获取reviews
          if (product.reviews == "")
            product.reviews = helper.regex(regex.search.html.reviews1, list);
          //       if(product.reviews =="") product.reviews = helper.regex(regex.search.html.reviews2, list);
          //       if(product.reviews =="") product.reviews = helper.regex(regex.search.html.reviews3, list);
          // if(product.reviews =="") product.reviews = helper.regex(regex.search.html.reviews4, list);
          product.sellersUsed = helper.regex(
            regex.search.html.sellersUsed,
            list
          ); //获取sellersUsed
          product.sellersNew = helper.regex(regex.search.html.sellersNew, list); //获取sellersNew
          if (product.sellersNew == "") product.sellersNew = 1; //当sellersNew为空时的赋值
          if (product.sellersUsed == "") product.sellersUsed = "-"; //当sellersUsed为空时的赋值
          //站点需要对价钱进行处理
          if (
            self.amazon.hostname == "www.amazon.de" ||
            self.amazon.hostname == "www.amazon.fr" ||
            self.amazon.hostname == "www.amazon.es" ||
            self.amazon.hostname == "www.amazon.it"
          ) {
            product.price = product.price.replace(".", "");
            product.price = product.price.replace(/\s/g, "");
            product.price = helper.replaceDot(product.price);
            if (product.rating != 0)
              product.rating = helper.replaceDot(product.rating);
          }
          setTimeout(
            product.getProductDetails.bind(product),
            50 * scrapeDelay++
          ); //获取详情页的sales revenue
          //获取全部的products的数据
          self.multiPage.push(product);
          self.nowProducts.push(product); //获取全部的products的数据
        }
      }
      if (!self.keywords) {
        var items = [];
        $.each(self.multiPage, function (index, detail) {
          items[index] = {};
          items[index].title = detail.title;
        }); //详情页获取关键词
        $.ajax({
          type: "post",
          url: "https://www.amztracker.com/get_main_keywords.php",
          data: { items: items },
          cache: false,
          success: function (e) {
            var e = JSON.parse(e);
            if (e.code == 1) self.keywords = e.data[0];
          },
        });
      }

      if (refresh == undefined) self.pageNext++;
    },
  });
};

UnicornSearch.prototype.storeResults = async function () {
  let value = await this.get()
  //发送消息保存数据
  chrome.runtime.sendMessage(
    { action: "saveSearch", search: value },
    function (response) { }
  );
};

Object.defineProperties(UnicornSearch.prototype, {
  ajaxCompleteCallback: {
    //赋值函数
    get: function () {
      return this.ajaxCompleteCallbackFunction;
    },
    set: function (callback) {
      this.ajaxCompleteCallbackFunction = callback;
      this.products.setAjaxCompleteCallback(callback);
    },
  },

  activeAjaxCalls: {
    //请求次数
    get: function () {
      var ajaxCount = this.xhrPool.length;

      $.each(this.products, function (index, product) {
        ajaxCount += product.xhrPool.length;
      });

      return ajaxCount;
    },
  },
});
